library(lidR)
library(lidRplugins)
library(dplyr)
library(sf)
library(stats)

# upload point cloud data
setwd("......")
las <-  readLAS("testPCDdata/16-2-summer-all.las")
las_f <-  readLAS("testPCDdata/16-2-summer-all.las", filter = "-drop_z_below 2")
 
# CHM generate & smoothing
chm_p2r_05 <- grid_canopy(las, 0.1, p2r(0.2, na.fill = tin()))
kernel <- matrix(1,7,7)
chm_p2r_05_smoothed <- terra::focal(chm_p2r_05, w = kernel, fun = median, na.rm = TRUE)
col <- height.colors(50)
## example of ITD (Local Maximum)
ttops_test <- locate_trees(chm_p2r_05_smoothed,lmf(6))
plot(chm_p2r_05_smoothed,col=col); plot(sf::st_geometry(ttops_test), add = T, pch=3)

# upload seed point from ITD
dat_MF <- read.table("test_SeedPointsFromMeanshift.txt")
len = nrow(dat_MF)
treeID = c(1:len)
treeID
## create some dummy data, coordinates for 3 squares
df <- data.frame(x = dat_MF$V1, 
                 y = dat_MF$V2, 
                 treeID = treeID,
                 stringsAsFactors = T)
# make the data frame spatial (as points)
tops_winter <- st_as_sf(df, coords = c("x", "y"))
ttops <- locate_trees(chm_p2r_05_smoothed,lmf(5.5))
st_crs(tops_winter) <- st_crs(ttops)

# silva2016 in Seed-based ITS
## time calculate
timestart<-Sys.time()
las_f <- segment_trees(las_f, silva2016(chm_p2r_05_smoothed, tops_winter))
timeend<-Sys.time()
runningtime<-timeend-timestart
print(runningtime)
plot(chm_p2r_05_smoothed,col=col); plot(sf::st_geometry(tops_winter), add = T, pch=3)
plot(las_f, bg = "white", size = 4, color = "treeID")

# Kmeans in Seed-based ITS
X <- data.matrix(las_f$X)
Y <- data.matrix(las_f$Y)
dat <- cbind(X,Y)
## add seed points
center_x <- data.matrix(dat_MF$V1)
center_y <- data.matrix(dat_MF$V2)
centers <- cbind(center_x, center_y)
KM <- kmeans(dat,centers)
plot(dat, col = KM$cluster)
points(KM$centers, col = 1:100, pch = 8, cex = 2)

# dalponte2016 with and without Seed-based ITS 
## Seed-based ITS
algo1 <- dalponte2016(chm_p2r_05_smoothed, tops_winter, max_cr = 100, th_tree = 15, th_seed = 0.45, th_cr = 0.55)
## No Seed-based ITS
ttops_test <- locate_trees(chm_p2r_05_smoothed,lmf(6))
algo2 <- dalponte2016(chm_p2r_05_smoothed, ttops_test, max_cr = 100, th_tree = 15, th_seed = 0.45, th_cr = 0.55)
## time calculate
timestart<-Sys.time()
las_f <- segment_trees(las_f, algo1) # segment point cloud
timeend<-Sys.time()
runningtime<-timeend-timestart
print(runningtime)
plot(las_f, bg = "white", size = 4, color = "treeID") # visualize trees
plot(chm_p2r_05_smoothed,col=col); plot(sf::st_geometry(ttops_test), add = T, pch=3)

# li2012 in No Seed-based ITS
algo <- li2012(R = 5.5, Zu = 15, hmin = 10, speed_up = 50)
## time calculate
timestart<-Sys.time()
las_f <- segment_trees(las_f, algo, attribute = "IDli")
timeend<-Sys.time()
runningtime<-timeend-timestart
print(runningtime)
plot(las_f,  bg = "white", size = 4, color = "IDli", pal = pastel.colors(200))

# layerstacking ITD in No Seed-based ITS
## time calculate
timestart<-Sys.time()
ttops = locate_trees(las_f,LayerStacking(start = 2, ws1 = 7, ws2 = 3.5, buf_size = 0.5, hardwood = FALSE, hmin = 15))
timeend<-Sys.time()
runningtime<-timeend-timestart
print(runningtime)
col <- height.colors(50)
plot(chm_p2r_05_smoothed,col=col); plot(sf::st_geometry(ttops), add = T, pch=3)

# ptrees in No Seed-based ITS
k = c(1500,750)
ttops = locate_trees(las_f,ptrees(k, hmin = 15))
## time calculate
timestart<-Sys.time()
las_f = segment_trees(las_f,ptrees(k, hmin = 15))
timeend<-Sys.time()
runningtime<-timeend-timestart
print(runningtime)
col <- height.colors(50)
plot(chm_p2r_05_smoothed,col=col); plot(sf::st_geometry(ttops), add = T, pch=3)
